<?php
/**
 * Controller for handling api functionality for sourcemap.
 *
 * @version 0.8.0
 * @author sourcemap@media.mit.edu
 * @package sourcemap
 * @subpackage controllers
 * @uses AuthModel ObjectsModel TransportModel
 */
class API extends SM_Controller {	
	public function API() {
		parent::SM_Controller();
		$this->load->model(Array('ObjectsModel', 'AuthModel', 'TransportModel'));		
	}
	
	/**
	 * Default controller, presents information about the api.
	 */
	public function index() {		
		$obj = $this->ObjectsModel->getRandomObject();
		if ($obj) { $this->data("randomObject", $obj); }
		else { $this->data("randomObject", "{object id}"); }
			
		$this->style(Array('info.css'));
		$this->display("API", "api_view");			
	}
	
	/**
	 * Produces json representation of a sourcemap.
	 * @param $slug string The sourcemap to json.
	 */
	public function json($slug=null) {
		if($slug == null) { $this->output->set_header("HTTP/1.1 400 Bad Request"); }
		else { 
			header('Content-type: application/json');
			header('Content-Disposition: attachment; filename="sourcemap-'.$slug.'.json"');
			$this->output->set_output($this->_getJsonObject($slug)); 
		}
	}
	
	/**
	 * Return the embodied carbon for the slug.
	 * @param $slug string The sourcemap to return.
	 * @todo implement.
	 */
	public function embodiedcarbon($slug) { }
	
	/**
	 * Produces the transport carbon of a trip after geocoding.
	 * @param $method string The transportation method.
	 * @param $placefrom string The origin placename.
	 * @param $placeto string The destination placename.	
	 * @todo implement
	 */
	public function transportcarbon($method, $placefrom, $placeto) {}
		
	/**
	 * Produces json list of transport methods.
	 */
	public function transportmethods() {
		$transports = $this->TransportModel->getSimpleTransportList();
		foreach($transports as $transport) {
			foreach($transport as $key => &$value) {
				if($key == "name") { $value = url_title($value);}				
			}
		}
		$this->output->set_output(json_encode($transports));
	}
	
	/**
	 * Produces kml for a sourcemap.
	 * @param $slug The sourcemqp to return.
	 * @uses kml
	 */
	public function kml($slug=null) {
		$this->load->helper('kml');
		
		if($slug == null) { $this->output->set_header("HTTP/1.1 400 Bad Request"); }
		else { 
			$object = $this->ObjectsModel->getObject($slug);
			if($object == null) {  $this->output->set_header("HTTP/1.1 400 Bad Request"); } 
			else {
				$this->output->set_header("Content-type: application/vnd.google-earth.kml+xml");
				$this->output->set_header('Content-Disposition: attachment; filename="sourcemap-'.$object->name.'.kml"');
				$this->output->set_output($this->_getKMLObject($object)); 
			}
		}
	}
	
	/**
	 * Allows the api creation of a sourcemap.
	 * @param $key the api key representing the account.
	 * @todo this is not currently implemented.
	 */
	public function create($key) {
		if($this->AuthModel->checkAPIKey($key)) {
			$object = json_decode($this->input->post("sourcemap"));
			echo $object->name;
		}
		else { $this->output->set_header("HTTP/1.1 403 Forbidden"); }
	}
	
	/**
	 * Allows the api updating of a sourcemap.
	 * @param $key The api key representing the account.
	 * @todo this is not currently implemented.
	 */
	public function update($key) {
		if($this->AuthModel->checkAPIKey($key)) {
			$object = json_decode($this->input->post("sourcemap"));
			echo $object->name;
		}
		else { $this->output->set_header("HTTP/1.1 403 Forbidden"); }
	}
	
	/**
	 * Encodes a sourcemap as json.
	 * @param $slug The sourcemap to return.
	 */
	private function _getJsonObject($slug=null) {
		$object = $this->ObjectsModel->getObject($slug);
		if($object == null) {  $this->output->set_header("HTTP/1.1 400 Bad Request"); } 
		else { return json_encode($object); }
	}
	
	/**
	 * Encodes a sourcemap as kml.
	 * @param $object The raw sourcemap object.
	 * @uses kml
	 */
	private function _getKMLObject($object) {
		return kml_encode($object); 
	}
} // End API