<?php
/**
 * Controller for initial load of sourcemap.
 *
 * @version 0.8
 * @author sourcemap@media.mit.edu
 * @package sourcemap
 * @subpackage controllers
 */
class Home extends SM_Controller {
	
	public function Home() {
		parent::SM_Controller();
		$this->load->model(Array('ObjectsModel', 'AuthModel', 'CollectionsModel', 'FeatureModel'));
		$this->load->helper();
	} 
	
	/**
	 * Default controller, loads the home screen.
	 */
	public function index() {
		$this->load->helper(Array('text','string','time'));
		$this->load->library('simplepie');
		
		$this->data('objectsArray', $this->FeatureModel->getFeaturedObjects(5, null, 'timemodified DESC'));
		
		$this->data('collections', $this->CollectionsModel->getCollectionsInRange(10, 0));
		$this->view('collectionsDisplay', 'collections/collectionslist_view');
		$this->view("objectsDisplay", "objects/objectslist_view");
		$this->contextual(Array('uservoice', 'news', 'highlightedmaps', 'quickticket'));
		$this->style(Array('objectlist.css','collections.css','home.css'));
		$this->script(Array('jquery/easyslider1.7.js', 'home.js'));	
		
		$this->data('pagedescription', 'Sourcemap is a place to create open supply chains for products, food, and travel using a shared carbon catalog and life cycle assessment (LCA) tools.');
		$this->display("Sourcemap - Open Supply Chains & Carbon Footprint", "home_view");
	}
	
	/**
	 * Error controller, loads when pages aren't found.
	 */
	public function page_not_found($reroute=null, $var=null) {
		if(!(isset($reroute))) {
			$this->data('url', $this->input->post("originalURL"));
			$this->style(Array('info.css'));
			$this->display("Page not found", "error_view");
		}
		else {
			if($reroute == "obj") {
				$object = $this->ObjectsModel->getObject($var);
				redirect($object->type."/".$object->slug);
			}
			if($reroute == "map") {
				$object = $this->ObjectsModel->getObject($var);
				redirect("map/embed/".$object->slug);
			}
		}
	}
	
	public function sourcemapinclude() {
		$this->display("Sourcemap Header", "sourcemapinclude_view");
	}
	/**
	 * Bug Submission controller, handle bug submission.
	 */
	public function bug() {
		$this->load->helper(Array('bug'));
		file_bug($this->input->post('bugtitle'), $this->input->post('bugdescription'));
		redirect($this->input->post('currenturl'));
	}
	
	/**
	 * Status controller, reports the status of this installation of sourcemap.
	 * @todo The status controller should be a real page.
	 */
	public function status() {
		print("Sourcemap Status:"); 
		
		print($this->config->item('deploystatus')."-");
		print($this->config->item('version')."-");
		print("OK");		
	}
	
	public function vcard() {
		$this->display("Vcard", "vcard_view");
	}
}