<?php
/**
 * Controller for displaying/editing/adding parts.
 * 
 * @version 0.8
 * @author sourcemap@media.mit.edu
 * @package sourcemap
 * @subpackage controllers
 */
class Parts extends SM_Controller{		
	function Parts(){
		parent::SM_Controller();
		$this->load->model(Array('PartsModel', 'CatalogueModel', 'AuthModel', 'CommentsModel', 'ObjectsModel'));		
	}
	
	/**
	 * Default controller, loads a listing of parts
	 * @param $sort how you want to sort the part list (defaults to name)		 
	 */
	function index($sort="name"){		
		$this->load->helper(Array('string', 'text'));
		$this->load->library('pagination');			
		
		$partsort = $this->getPartSort($sort);
		
		$range = $this->uri->segment(4); 
		$pageper = 30; 
		$pagetotal = $this->PartsModel->getPartCount();		
			
		$this->paginate($pageper,site_url() . '/parts/sort/'.$sort."/",$pagetotal,4);
	
		if($this->uri->segment(4) <= 1) { $this->data('infomessage', true); }						
		$this->data('partsArray', $this->PartsModel->getParts(30, $range, $partsort));
		$this->data("sort", $sort);
					
		$this->script(Array('jquery/jquery.heatcolor.js', 'parts.js'));
		$this->style(Array('parts.css'));			
		$this->view("partsDisplay", "parts/partslist_view");
		$this->contextual(Array('quickticket'));
		
		$this->display("Carbon Catalogue", "parts_view");
	}
	
	/**
	 * Loads a single part representation.
	 * @param mixed $mid Can be name or number, name expects a mapping of ':' to ', ' and '-' to ' '.
	 */
	function single($mid=null) {	
		if($mid == null) { show_404(); }		
	
		if(!(is_numeric(substr($mid, 5)))) {
			$part = $this->PartsModel->getPartbySlug($mid);
		} else { $part = $this->PartsModel->getPartbyID(substr($mid, 5));}
		if( $part == null) { show_404();}
				
		$this->data('part', $part);

		$this->data("loggedin", $this->AuthModel->checkUserLogin());			
		$this->data("comments", $this->CommentsModel->getComments("part", $part->id));
				
		$this->script(Array('jquery/jquery.heatcolor.js', 'parts.js'));

		$this->style(Array('parts.css'));			
		$this->view("partsDisplay", "parts/partsingle_view");
		$this->contextual(Array('quickticket'));
		
		$this->display($part->name, "parts_view");
	}
	
	/**
	 * Save of current suggested object (passed by json post)
	 */
	public function save() {
		$suggestPart = json_decode($this->input->post("data"));
		$this->PartsModel->savePart($suggestPart);			
	}	
	
	/**
	 * Approves current suggested object
	 */
	public function approve() {
		$this->load->helper('json'); 
		$approveID = $_POST["data"];
		$this->PartsModel->approvePart($approveID);			
	}
		
	/**
	 * Maps part sort short name to sql representation.
	 */
	private function getPartSort($sort) {
		if($sort == "name") { $partsort = 'name ASC'; }
		if($sort == "category") { $partsort = 'class ASC, name ASC'; }			
		if($sort == "emissions") { $partsort = 'co2_eq ASC'; }
		if($sort == "namedesc") { $partsort = 'name DESC'; }
		if($sort == "categorydesc") { $partsort = 'class DESC, name ASC'; }			
		if($sort == "emissionsdesc") { $partsort = 'co2_eq DESC'; }
		return $partsort;
	}
	
	/*Gets the part location from the model*/
	public function getPartLocations() {
	  $data = json_decode($this->input->post("data"));
	  $get_part_location= utf8_decode($data->part_location);
	  $material_name = utf8_decode($data->name);
	  $part_location_array = $this->ObjectsModel->getPartLocationData($get_part_location, $material_name);
	  $this->output->set_output(json_encode($part_location_array));
	}	
}