<?php  if (!defined('BASEPATH')) exit('No direct script access allowed');

//Handles Google Base authentication, data retrieval and parsing, sorting of parsed data and data submission
//Completed: Data retrieval/parsing, sorting of parsed data
class GBase {
	
	private $xml = "";
	private $submitData = "";
	private $url = "";
	private $query = "";
	private $entries = array();
	private $currElement = "";
	private $foundEntry = false;
	
	//TODO: Load the following values from a suitable controller or config file:
	public $blackList = array('eBay');	
	public $whiteList = array('Best Buy', 'Walmart', 'Office Depot', 'Sam\'s Club', 'OfficeMax', 'Target', 
					   'Amazon.com', 'Dell', 'Toys\"R\"Us', 'Fry\'s Electronics', 'RadioShack.com',
					   'KMART', 'Sears', 'Newegg.com', 'fye.com', 'The Home Depot');
	const authToken = "CIm7hJWhAhDn2P6S_f____8B";
	const devKey = "ABQIAAAAlmzte9d_6ohYlBURfzuD3hRBayFfekbyzc5Gccb_xCLTXjmmThR1xcohRarUo_EUm277frS2-Q8JuQ";
	const itemsURL = "http://www.google.com/base/feeds/items/";
	const snippetsURL = "http://www.google.com/base/feeds/snippets/";

	public function GBase($config = array()) {
	}

	/*
	* Basic Accessors and Mutators
	*/
	
	public function setURL($url) {
		//Need our URLs to be formatted with a '/' at the end.
		if($url != "" && $url[strlen($url) - 1] != "/") {
			$this->url = $url."/";
		}
		else
			$this->url = $url;
	}
	
	public function getURL() {
		return $this->url;
	}
	
	public function setQuery($query) {
		$this->query = $query;
	}
	
	public function getQuery() {
		return $this->query;
	}
		
	//Return the Raw XML. May be useful for debugging, data analysis, etc.
	public function getRawXMLData() {
		return $this->xml;
	}
	
	//Return the parsed XML entries
	public function getStructuredData() {
		return $this->entries;
	}
	
	/*
	* Methods for processing, submitting, retrieving, and sorting data
	*/
	
	//Get the Raw XML of a feed. Its data is placed in $this->xml.
	//Parse the XML into $this->entries using parseGBaseFeed()
	public function retrieveFeed() {
		$fullURL = $this->url."?".$this->query;
		
		$ch = curl_init();
		curl_setopt($ch, CURLOPT_URL, $fullURL);
	  	curl_setopt($ch, CURLOPT_RETURNTRANSFER, true); //Return value rather than variable dump

		//We're using the items feed, so we need to authenticate
	  	if($this->url == self::itemsURL) {
			curl_setopt($ch, CURLOPT_HTTPHEADER, array(
		    'Content-Type: application/atom+xml',
		    'Authorization: AuthSub token="'.trim(self::authToken).'"',
		    'X-Google-Key: key='.self::devKey
		  )); 
		}
	
		$this->xml = curl_exec($ch);
		curl_close($ch);
		
		//Now that we have some XML, parse it.
		$this->parseGBaseFeed();
	}
	
	private function populateItemData(&$itemData, $entries) {
		if(!$this->hasEmptyFields($itemData))
			return;
		$prices = array();
		
		foreach($entries as $entry) {
			$itemData['name'] = $this->tryStringSet($itemData['name'], $entry, 'title');
			$itemData['desc'] = $this->tryStringSet($itemData['desc'], $entry, 'content');
			$itemData['url'] = $this->tryStringSet($itemData['url'], $entry, 'alternate');
			$itemData['imgsrc'] = $this->tryStringSet($itemData['imgsrc'], $entry, 'g:image_link');
			$itemData['upc'] = $this->tryStringSet($itemData['upc'], $entry, 'g:upc');
			$itemData['weight'] = $this->tryStringSet($itemData['weight'], $entry, 'g:weight');
			$itemData['length'] = $this->tryStringSet($itemData['length'], $entry, 'g:length');
			$itemData['width'] = $this->tryStringSet($itemData['width'], $entry, 'g:width');
			$itemData['height'] = $this->tryStringSet($itemData['height'], $entry, 'g:height');
			$itemData['loc'] = $this->tryStringSet($itemData['loc'], $entry, 'g:location');
			
			$currPrice = 0;
			$currPrice = floatval($this->tryStringSet($currPrice, $entry, 'g:price'));
			if($currPrice != 0)
				$prices[] = $currPrice;
		}
		if(!empty($prices) && $itemData['medianprice'] == '') {
			$itemData['medianprice'] = strval(round($this->median($prices),2));
		}
	}
	
	//Submits an entry to Google Base. Takes an array of attribute => value pairs and 
	//an item type (e.g. 'products').
	public function submitItem($entry = array(), $itemType) {
		if(empty($entry))
			return;
		
	  	$this->submitData  = "<?xml version='1.0' encoding='utf-8'?>\n";
	  	$this->submitData .= "<entry xmlns='http://www.w3.org/2005/Atom' xmlns:g='http://base.google.com/ns/1.0'>\n";
	  	$this->submitData .= "<category term='$itemType' scheme='http://base.google.com/categories/itemtypes'>\n";
	  	$this->submitData .= "<title type='text'>".$entry['title']."</title>\n";
		$this->submitData .= "<g:item_type>".strtolower($itemType)."</g:item_type>\n";
		foreach($entry as $key => $value) {
			if($key != 'title' && $key != 'content') {
				$this->submitData .= "<g:$key>$value</g:$key>\n";
			}
		}
	  	$this->submitData .= "<content>".$entry['content']."</content>\n";
	  	$this->submitData .= "</category></entry>\n";
		
		//Try submitting data to Google Base
		return $this->postEntry();
	}
	
	//Posts raw XML to Google Base. Unlike submitItem(), it does not construct
	//the XML for you. Can be used for debugging/diagnostic purposes.
	public function submitRawXML($xml) {
		$this->submitData = $xml;
		return $this->postEntry();
	}
	
	//Submits data in $this->submitData to Google Base. Called by submitItem()
	private function postEntry() {
	  	$ch = curl_init();

	  	//Configure cURL
	  	curl_setopt($ch, CURLOPT_URL, self::itemsURL);
	  	curl_setopt($ch, CURLOPT_POST, true);
	  	curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
	  	curl_setopt($ch, CURLOPT_FAILONERROR, false);
		//Authenticate
		curl_setopt($ch, CURLOPT_HTTPHEADER, array(
	    	'Content-Type: application/atom+xml',
	    	'Authorization: AuthSub token="'.trim(self::authToken).'"',
	    	'X-Google-Key: key='.self::devKey
	  		));
		//Attach XML
	  	curl_setopt($ch, CURLOPT_POSTFIELDS, $this->submitData);

	  	$result = curl_exec($ch);
		if($result == false) {
			echo curl_error($ch);
		}
	  	curl_close($ch);
		return $result;
	}
	
	
	//Returns a subset of $this->entries whose entries have attribute $attr that matches
	//one of the values in the $values array. Behavior is opposite of $this->getStructuredDataNotMatching()
	public function getStructuredDataMatching($attr, $values = array()) {
		if(empty($this->entries)) {
			return array(); //No feed has been processed yet
		}
		
		if(empty($values) || $attr == '')
			return $this->entries; //User wasn't specific enough, so we'll return all entries
			
		else {
			$subset = array();
			//Find all entries whose attribute $attr is in the $values array
			foreach($this->entries as $entry) {
				foreach($values as $value) {
					if($this->safelyGetAttribute($entry, $attr) == $value) {
						$subset[] = $entry; //Add this $entry to the returned array
						break; //Prevent duplicates if there are duplicates in $values array
					}
				}
			}
		return $subset;
		}
	}
	
	//Returns a subset of $this->entries whose entries have attributes $attr not matching
	//values in $values. Behavior is opposite of $this->getStructuredDataMatching()
	public function getStructuredDataNotMatching($attr, $values = array()) {
		if(empty($this->entries)) {
			return array(); //No feed has been processed yet
		}
		
		if(empty($values) || $attr == '')
			return array(); //User wasn't specific enough, so we'll return no entries
			
		$matchingEntries = $this->getStructuredDataMatching($attr, $values);
		$subset = array();

		foreach($this->entries as $entry) {
			$foundMatch = false;
			foreach($matchingEntries as $match) {
				if($entry == $match) {
					$foundMatch = true;
					break;
				}
			}
			//Add non-matching entries to the returned array.
			if($foundMatch == false)
				$subset[] = $entry;
		}
		return $subset;
	}
	
	//Returns a sorted version of $this->entries. The first entries in the list will have an
	//attribute $attr that matches one of the values in $values. Note: The order of the values in
	//$values will not necessarily affect the order of the sort.
	public function sortedStructuredDataBy($attr, $values = array()) {
		if(empty($this->entries)) {
			return array(); //No feed has been processed yet
		}
		
		if(empty($values) || $attr == '')
			return $this->entries; //User wasn't specific enough; no need to sort
			
		$sortedList = array();
		$bestEntries = $this->getStructuredDataMatching($attr, $values);
		$otherEntries = $this->getStructuredDataNotMatching($attr, $values);
		
		//Sorting will have no effect, so just return the whole set of entries
		if(empty($bestEntries) || empty($otherEntries))
			return $this->entries;
		
		//Populate the list from $bestEntries, then from $otherEntries
		foreach($bestEntries as $best)
			$sortedList[] = $best;
		foreach($otherEntries as $other)
			$sortedList[] = $other;

		return $sortedList;
	}
	
	/*
	* Private library methods
	*/
	
	//Prevents array indexing errors. Will try to return $container[$attr] if it exists.
	//Returns $defaultVal otherwise
	private function safelyGetAttribute($container, $attr, $defaultVal = false) {
		if(isset($container[$attr]))
			return $container[$attr];
		else return $defaultVal;
	}
	
	//Handles parsing of the Google Base Feed
	/*NOTE: The parser will build the array $this->entries.
	* Each element of $this->entries is itself an array
	* that contains data on a particular item in the
	* Google Base feed that was returned by Google.
	* Use $this->entries[i][name] with i as an index to the entry
	* you want and name as a string with the name of the
	* field you'd like to retrieve. Here are some common fields:
	* 'title' => name of item, 'name' => Author of entry,
	* 'content' => item description, 'g:upc' => UPC code
	* 'g:price' => "$XXX.XX USD", 'g:image_link' => link to image
	* of item. 'g:mpn' => Model number. 'alternate' => URL of item
	* Please note that not every entry will contain every 
	* one of these fields, so don't index into the array unless 
	* you know a value exists at that index.
	*/
	private function parseGBaseFeed() {
		//Parse XML data using handler functions
		//Data is placed in the entries private variable
		//Raw XML data is not modified
		
		//Reset variables and create the parser
		$this->entries = array();
		$this->currElement = "";
		$this->foundEntry = false;
		$xml_parser = xml_parser_create();
		
		//Pass in our callback functions with a reference to $this
	    xml_set_element_handler($xml_parser, array($this, 'startElement'), array($this, 'endElement'));
	    xml_set_character_data_handler($xml_parser, array($this, 'characterData'));
	    //Parse $xml into $entries
	    xml_parse($xml_parser, $this->xml);
	    xml_parser_free($xml_parser);
	}
	
	/*
	* XML parser callback functions
	*/
	
	//Deals with tag opens
	private function startElement($parser, $name, $attrs) {
  		$this->currElement = $name;
  		if($this->currElement == "ENTRY") {
    		$this->foundEntry = true;
			//Open tag: instantiate an empty array at the current index in $this->entries
    		$this->entries[count($this->entries)] = array();
  			} else if($this->foundEntry && $this->currElement == "LINK") {
				//The open tag is actually a link. Don't open a new tag, but instead add the data
    			$this->entries[count($this->entries) - 1][$attrs["REL"]] = $attrs["HREF"];
  			}
	}
	
	//Deals with tag closes
	private function endElement($parser, $name) {
  		if($name == "ENTRY") {
			//Close tag
    		$this->foundEntry = false;
  		}
	}
	
	//Deals with data within tags
	private function characterData($parser, $data) {
  		if($this->foundEntry) {
			//Add contents of current tag to the current index of $this->entries
    		$this->entries[count($this->entries) - 1][strtolower($this->currElement)] = $data;
  		}
	}
	
	private function hasEmptyFields($x = array()) {
		foreach($x as $item)
			if($item == '')
				return True;
		return False;
	}

	//Find the median of a set of values	
	private function median($data = array()) {
		if(empty($data)) 
			return NULL;
		else if(count($data) == 1) 
			return $data[0];
		else {
			sort($data, SORT_NUMERIC);
			$mid = intval((count($data) / 2));
			//Deal with cases separate in which the list is odd or even in length
			return (count($data) % 2 != 0) ? $data[$mid] : floatval(($data[$mid] + $data[$mid - 1]) / 2);
		}	
	}
	
	//Returns $source[$index] if it exists, otherwise returns $dest
	//Safeguards against bad indices and overwriting good data
	//Usage: $d = tryStringSet($d, $myArray, $arrayIndex)
	private function tryStringSet($dest, $source, $index) {
		if(isset($source[$index]) && $dest == "")
			return $source[$index];
		else 
			return $dest;
	}
	
	public function queryUPC($upc, $numResults, $feed) {
		
		//Build the structured UPC query
		$bq = 'bq=[upc(text):"'.$upc.'"]'.'&max-results='.$numResults;
	
		$this->setQuery($bq);
		if($feed == "items") {
			$this->setURL(self::itemsURL);
		} else {
			$this->setURL(self::snippetsURL);
		}
		$this->retrieveFeed();
		$entries = $this->getStructuredData();
		
		$itemInfo = array('name' => '', 'desc' => '', 'upc' => '', 'url' => '', 'imgsrc' => '', 
						  'medianprice' => '', 'weight' => '', 'loc' => '', 'length' => '',
						  'width' => '', 'height' => '');
						
		$wPool = $this->getStructuredDataMatching('name', $this->whiteList); //Pool from trusted data authors
		$bPool = $this->getStructuredDataMatching('name', $this->blackList); //Pool from untrusted authors
		$gPool = $this->getStructuredDataNotMatching('name', array_merge($this->whiteList, $this->blackList)); //Everyone else
		
		//Fill up $itemInfo with data from trusted, unknown, and finally untrusted authors
		$this->populateItemData($itemInfo, $wPool);
		$this->populateItemData($itemInfo, $gPool);
		$this->populateItemData($itemInfo, $bPool);
		
		return $itemInfo;
	}
}
?>