<?php

/**
 * Representation of objects as model, handles transactions for
 * objects so they can be listed or edited. 
 * 
 * @author Sourcemap Team
 * @package sourcemap
 * @subpackage models
 */



class FeatureModel extends Model {

	function FeatureModel(){
		parent::Model();	
		$this->load->model(Array('ObjectsModel'));
		$this->load->library('Log');
		// set visible instead of where visiblity, where private
	}


/**
 * Returns the objects from the featured category
 * @limit int $limit (how many featured objects)
 * @sort timemodified in descending, category and name in ascending.
 */
function getFeaturedObjects($limit = null, $offset = null, $sort = 'timemodified DESC, category ASC, name ASC'){
	$this->log->write_log('info', "getting featured objects");
	$this->db->cache_on();
	
	$this->db->select('*');		
	$this->db->from('objects_complex');
	$this->db->where('flag !=', 'temporary');		
	$this->db->where('name !=', '');
	$this->db->where('visibility !=', 'private');
	$this->db->where('category', 'featured');
		
	$this->db->limit($limit, $offset);
	$this->db->order_by("", "RANDOM");
	
	$objects = $this->db->get();
	$objects = $objects->result();
	return $this->ObjectsModel->decorateObjects($objects);

}


/**
 * Gets array of objects from the objects_complex
 * @param int $limit =10
 * @order parts count in descending order
 */
function getComplexObjects($limit=10){
	$this->db->select("objects.*, parts.*", false);
	$this->db->from("objects_complex objects");
	$this->db->where('objects.visibility !=', 'private');		
	$this->db->join("(select count(oid) as count,oid from parts_instantiated group by oid) as parts", "parts.oid = objects.oid", "inner");
	$this->db->order_by("parts.count desc");
	$this->db->limit($limit);
	
	$objects = $this->db->get();
	$objects = $objects->result();
	return $objects;
}

/**
 * Return array of objects from objects_complex and favorites
 * @limit int $limit =10
 */
function getFavoritedObjects($limit=10){
	$this->log->write_log('info', "getting favorited objects");
	$this->db->select("objects.*, favorites.*", false);
	$this->db->from("objects_complex objects");
	$this->db->where('objects.visibility !=', 'private');
	$this->db->join("(select count(oid) as count,oid from favorites group by oid) as favorites", "favorites.oid = objects.oid", "inner");
	$this->db->order_by("favorites.count desc");
	$this->db->limit($limit);
	
	$objects = $this->db->get();
	$objects = $objects->result();
	return $objects;
}


/**
 * Returns array of objects from objects_complex and comments tables
 * @order_by comments count in descending order
 * @limit int $limit=10 (these many objects are returned).
 */
function getCommentedObjects($limit=10){
	$this->log->write_log('info', "getting commented objects");
	$this->db->select("objects.*, comments.*", false);
	$this->db->from("objects_complex objects");
	$this->db->where('objects.visibility !=', 'private');
	$this->db->join("(select count(refid) as count,refid from comments group by refid) as comments", "comments.refid = objects.oid", "inner");
	$this->db->order_by("comments.count desc");
	$this->db->limit($limit);
	
	$objects = $this->db->get();
	$objects = $objects->result();
	return $objects;
}

}
?>