/**
 * @fileoverview  Sourcemap.Editor: Main component to handle viewing and editing of Sourcemaps.
 *
 * @version 0.8
 * @author sourcemap@media.mit.edu
 * @package sourcemap
 * @subpackage js
 */

/**
 * Create a new Sourcemap instance.
 * @see  Sourcemap 
 */
if (typeof(Sourcemap) == 'undefined' ) {Sourcemap = {}; }

/**
 * @see  #initializeEditMode
 * @see  #initializeTooltips
 */
Sourcemap.Editor = {
    init: function() {

	/**
	* Triggers edit mode of objects by switching fields and enabling interaction.
	*/
	this.processHashEvent = function(hash) {
		if(hash == "#edit") {
			this.initializeEditMode();
			if(Sourcemap.Template.usechooser) { this.showPartChooser();}
		}			
		else if(hash == "#new") {
			this.initializeEditMode();
			if(Sourcemap.Template.usechooser) { this.showPartChooser();}
			this.initializeTooltips();				    
		}			
	};
	
	/**
	 * Intialize all the functions
	 * Make all the fields editable 
	 * @see  #setupPartHandlers
	 * @see  #setupEndoflifeHandlers
	 * @see  #setupProcessHandlers
	 */
 	this.initializeEditMode = function() {	
	// Make Objects Editable
	    $("#object-name").html('<input class="input-object-name" value="'+$("#object-name").text()+'">'); 
	    $("#object-description").html('<textarea class="input-object-description">'+$("#object-description").html()+'</textarea>');
	    $("#object-origin").html('<input class="input-object-origin" value="'+$("#object-origin").text()+'">');
	    $("#object-destination").html('<input class="input-object-destination" value="'+$("#object-destination").text()+'">');
	    $("#object-via select").removeAttr("disabled");
	    
	    $( "#part-list").addClass("editmode");						    
	    $("div#partlist-shell").css("height","360px");	
	
	    // Make Parts Editable
	    $(".part-name").each(function() { $(this).html('<input class="input-name" value="'+$(this).text()+'">');});
	    $(".part-description").each(function() { $(this).html('<textarea class="input-description">'+$(this).html()+'</textarea>');});	
	    $(".part-weight").each(function() { $(this).html('<input class="input-weight" value="'+$(this).text()+'">');});
	    $(".part-via select").removeAttr("disabled");
	    $(".part-origin").each(function() { $(this).html('<input class="input-origin" value="'+$(this).text()+'">');});
	    $(".part-rawemissions").each(function() { $(this).html('<input class="input-rawemissions" value="'+$(this).text()+'">');});
	    			    			    
	    // Make Process Editable
	    $(".process-text").each(function() { 
					$(this).html('<input class="input-process-text" value="'+$(this).text()+'"/>');
				    });

	    $(".process-area").each(function() { 
			if($(this).parents(".process-details").find("span.process-unit").html() != "kg") {
				$(this).html('<input class="input-process-area" value="'+$(this).text()+'"/>');
			}
		});

	    $(".process-display").each(function() { 
					   $(this).html('<input class="input-process-display" value="'+$(this).text()+'"/>');
				       });
	    
	    //Make Endoflife Editable
	    $(".endoflife-display").each(function() { $(this).html('<input class="input-endoflife-display" value="'+$(this).text()+'">');});
	    
	    $(".endoflife-emissions").each(function() { $(this).html('<input class="input-endoflife-emissions" value="'+$(this).text()+'">');});


	    // Set handlers to Sourcemap.Object
	    $(".input-object-name").change(function() { 
		    objectConfig.setName($(this).val()); $(this).addClass("dirty");
		});
	    $(".input-object-description").change(function() { 
		    objectConfig.setDescription($(this).val()); $(this).addClass("dirty");
		});
	  
	    $(".input-object-origin").change(function() { 
		    objectConfig.setOrigin($(this).val()); $(this).addClass("dirty");
		});
	    $(".input-object-destination").change(function() { 
		    objectConfig.setDestination($(this).val()); $(this).addClass("dirty");
		});
	    $("#object-via-select").change(function() { 
		    objectConfig.setVia($("#object-via select :selected").text()); $(this).addClass("dirty");
		});
	    	        			    
	    Sourcemap.Editor.setupPartHandlers("");
            Sourcemap.Editor.setupProcessHandlers("");	    
	    Sourcemap.Editor.setupEndoflifeHandlers("");
	    Sourcemap.Editor.setupLocationComplete("");	      
	    Sourcemap.Editor.setupPermissions("");	    	    

	    window.onbeforeunload = function() { return "You are currently editing a sourcemap and you haven't saved your changes. If you want to save your changes, click the green 'Save Sourcemap' button before you leave."; };	 
	    
	    $("#permission-button").removeClass("hidden");

	    $("li.part-details").addClass("hover");
	    $(".part-details .removeaction").css("display","inline");
	    
	    $('input:checkbox').removeAttr("disabled"); 
	    $("#usekwh, #poweruselist, #transportlist").removeAttr("disabled"); 
	    $("div#summary-statistics .checkmessage, div#summary-statistics input[type='checkbox']").css("display","inline");
	    
	    $("#editsourcemapbutton").css("display","none");
	    $("#savesourcemapbutton").slideDown();
	    $("#addpartbutton, #partlist-menu").slideDown();

	    $(".list-expand").text("-");			
		$(".list").css("display","block");

	    
	    
  	};
	this.setupLocationComplete = function(specifier) {
		    //Auto Complete Part Location

		    $(".part-origin .input-origin").unbind("keyup");
		    $(".part-origin .input-origin").keyup(function(e) {
				 if(e.keyCode!=40 && e.keyCode != 13  && e.keyCode != 38 && e.keyCode != 27)  {
				     Sourcemap.Editor.filterLocate(e);
				 }
			});
			// Display all the locations for the part
			$(".list").unbind("click");
			$(".list").click(function(e) {
					     var part_locate ="";
					     var part_location_id = parseInt($(e.target).parents(".part-details").attr("id"));
					     var part_location_name = objectConfig.parts[part_location_id].name;
					     var saveData = { part_location: part_locate,
							      name: part_location_name };

					     var sendData = "data=" + JSON.stringify(saveData) + "";
					     $.post(Sourcemap.siteurl+"parts/getPartLocations", sendData, function(response) {
							var part_locations = eval(response);
							$(".ac_results").remove();

							// create a div and as a child to the parent of the input field (use append)	       
							var locate = $("li#"+part_location_id+".part-details .part-origin");
							locate.append('<div class="ac_results"><ul>');
							$.each(part_locations, function(i,e) {
								   // create one <li>location</li> per location inside div
								   $(".ac_results ul").append('<li>'+e.displaylocation+'</li>');

							       });
							locate.append('</ul></div>');

							Sourcemap.Editor.hoverLocate(locate.find("input.input-origin"));

						    });

					 });
	};
	

	/**
	 * To autocomplete the location in the input field by hover over function
	 * @param target. This is a classname where the location has to be display
	 */
	this.hoverLocate = function(target) {
	    $(target).parent().find(".ac_results ul li").hover(
		function(event) {
		    $(event.target).css('background', 'yellow');
		    $(target).val($(event.target).html());
		},
		function(event) {
		    $(event.target).css('background', '');
		});
	    $(".ac_results ul li").click(function(event) {
					     $(".ac_results").remove();
					 });

	    $(target).blur(function(event) {
			   $(".ac_results").remove(); 
		       });
	    
	};
	

	/**
	 * Do the ajax call to the server to get the locations
	 * Display them in the part-origin input field
	 * With keydown/keyup, different locations can be selected
	 * @param e, this is the target element
	 * @see  #hoverLocate 
	 */
	 this.filterLocate = function(e) {
	    


	    var part_locations_display = Array();
	    var part_location_id = parseInt($(e.target).parents(".part-details").attr("id"));
	    var part_location_name = objectConfig.parts[part_location_id].name;
	    var filterPartLocate = $(e.target).val();
	    var saveData = { part_location: filterPartLocate,
			     name: part_location_name };
	    
	    var sendData = "data=" + JSON.stringify(saveData) + "";
	    $.post(Sourcemap.siteurl+"parts/getPartLocations", sendData, function(response) {
                       var part_locations = eval(response);
		       $(".ac_results").remove();
		       
		       // create a div and as a child to the parent of the input field (use append)	       
		       $(e.target).parent().append('<div class="ac_results"><ul>');
		       $.each(part_locations, function(i,e) {
				  // create one <li>location</li> per location inside div
				  $(".ac_results ul").append('<li>'+e.displaylocation+'</li>');
				  part_locations_display = e.displaylocation;
			      });
		       $(e.target).parent().append('</ul></div>');
		       Sourcemap.Editor.hoverLocate(e.target);
		   });
	    
	    var currentSelection = null;
	    
	     
	     $(e.target).unbind("keydown");
	     
	     $(e.target).keydown(function(event) {
					
					var target = $(e.target);
					var ac = target.parent().find('.ac_results');
								
					if(ac.length == 0 ) { return; }
					if(event.keyCode==40) {
					    if(currentSelection==null) {
						currentSelection = ac.find("li")[0];
					    } else { 
						$(currentSelection).css('background', '');
						currentSelection = $(currentSelection).next();
						if(currentSelection.length == 0) {
						    currentSelection = ac.find("li")[0];
						} else {
						    currentSelection = currentSelection[0];
						}
					}
					    
					    $(currentSelection).css('background', 'yellow');  
					 					    
					} else if(event.keyCode==38) {
					 
					    if(currentSelection==null) {
						currentSelection = ac.find("li")[ac.find("li").length-1];
					    } else {
						$(currentSelection).css('background', '');
						currentSelection = $(currentSelection).prev();
						if(currentSelection.length == 0) {
						    currentSelection = ac.find("li")[ac.find("li").length-1];
						} else {
						    currentSelection = currentSelection[0];
						}
					    }
					    $(currentSelection).css('background', 'yellow');
					    
				    } else if(event.keyCode==13) {
						target.val($(currentSelection).html());
				    	$(".ac_results").remove();
					} else if (event.keyCode==27) {
					    $(".ac_results").remove();
					}
					else {
						$("ac.results").remove();
					}

				    });	

	};

	/**
	 * Stop all the functions which were enabled in edit mode.
	 * The fields are not editable and are all hidden
	 */
	this.stopEditMode = function() {	
	    $(".qTip").remove();
	    window.location.hash = "#";
    
	    $( "#ajaxnote" ).text("Saving Object..."); // TODO should become notes area.
	    $( "#part-list").removeClass("editmode");

	    $("#permission-button").addClass("hidden");
	    $("#permission-panel").css("display","none");

	    Sourcemap.Editor.closePartChooser();
	    
	    objectConfig.recalculateObjectValues();	
	    objectConfig.save();
	    
	    $(".list-expand").text("+");
	    $(".list-expand").parents(".part-details").removeClass("hover");
	    
	    
	    // Add input switch to unedit object
	    $(".input-object-name").each(function() { $(this).replaceWith($(this).val());});		
	    $(".input-object-description").each(function() { $(this).replaceWith(Sourcemap.Util.rhtmlspecialchars($(this).val()));});			
	    $(".input-object-permissions").each(function() { $(this).replaceWith($(this).val());});		
	    $(".input-object-origin").each(function() { $(this).replaceWith($(this).val());});		
	    $(".input-object-destination").each(function() { $(this).replaceWith($(this).val());});
	    
	    $(".input-object-via").attr("disabled", "disabled"); $(".input-object-via").removeClass("dirty");
	    $(".input-object-lifetime").each(function() { $(this).replaceWith($(this).val());});
	    
	    // Add input switch to unedit parts	
	    $(".input-name").each(function() { $(this).replaceWith($(this).val());});
	    $(".input-description").each(function() { $(this).replaceWith(Sourcemap.Util.rhtmlspecialchars($(this).val()));});
	    $(".input-weight").each(function() { $(this).replaceWith($(this).val());});
	    $(".input-via").attr("disabled", "disabled"); $(".input-via").removeClass("dirty");	
	    $(".input-origin").each(function() { $(this).replaceWith($(this).val());});
	    $(".input-rawemissions").each(function() { $(this).replaceWith($(this).val());});
	    
	    //$(".part-details .removeaction").css("display","none");
	    
	    // Add input switch to unedit process
	    $(".input-process-text").each(function() { $(this).replaceWith($(this).val());
						     });       
		// TODO Add check to see if unit == kg, if it doesn't don't print this.   
	    $(".input-process-area").each(function() { $(this).replaceWith($(this).val());
						     });          
	    $(".input-process-display").each(function() { $(this).replaceWith($(this).val());
						     });          

	   //Add input switch to unedit endoflife
	    $(".input-endoflife-display").each(function() { $(this).replaceWith($(this).val());
						     });          
	    $(".input-endoflife-emissions").each(function() { $(this).replaceWith($(this).val());
						     });          
		
	    
	    // Add capability to create parts
	    $("#private-button").unbind("click");
	    $("#editsourcemapbutton").slideDown();
	    $("#savesourcemapbutton").css("display","none");
	    $("#addpartbutton, #partlist-menu").css("display","none");
	    $("div#summary-statistics .checkmessage, div#summary-statistics input[type='checkbox']").css("display","none");		
	    $("div#partlist-shell").css("height","385px");
	    $('#partchooser-visualization').css("display","none");
	    $("input:checkbox").attr("disabled","true");
	    $("#usekwh, #poweruselist").attr("disabled", "true"); 
	};

	/**
	 * Call a function in the objects.js to add a process
	 * @param target_element, it is the location where the add process
	 * button is located.
	 * @see  #getProcessesForPart
	 */
	this.addProcessPart = function(target_element){

	    var part_id = parseInt($(target_element).parents(".part-details").attr("id"));
		if($(target_element).parents(".part-details").find(".process-details").length == 1) {
			elementid = $(target_element).parents(".part-details").find(".process-details").attr("id");
		} else {
			elementid = "process-0";
		}
	    var process_id = parseInt(elementid.substring(8));
		
	    Sourcemap.Editor.getProcessesForPart(part_id, process_id);
	    $("li#"+part_id+" .add-process").css("display","block");
	  };

	/**
	 * Call a function in the objects.js to ass an endoflife for a part
	 * @param target_element, it is the location where add endoflife button
	 * is located 
	 * @see  #getEndoflifeForPart
	 */
	this.addEndoflifePart = function(target_element) {
		var part_id = parseInt($(target_element).parents(".part-details").attr("id"));
		if($(target_element).parents(".part-details").find(".process-details").length == 1) {
			elementid = $(target_element).parents(".part-details").find(".endoflife-details").attr("id");
		} else {
			elementid = "endoflife-0";
		}
	    parseInt(elementid.substring(10));
	    Sourcemap.Editor.getEndoflifeForPart(part_id, endoflife_id, target_element);	    
	    $("li"+part_id+" .add-endoflife").css("display", "block");
	};
	
	/**
	 * Setup handlers for a part
	 * @param specifier, this points to "li #id"
	 * Call the associated function in objects.js for every change function
	 * @see  #intializeEditMode
	 */
	this.setupPartHandlers = function(specifier) {
	    // Set part handlers to Sourcemap.Object
	    
	    $(specifier + ".input-name").change(function() {
			objectConfig.setPartName(parseInt($(this).parents(".part-details").attr("id")), $(this).val()); 
			$(this).addClass("dirty");
		});
	    $(specifier + ".input-description").change(function() {
			objectConfig.setPartDescription(parseInt($(this).parents(".part-details").attr("id")), $(this).val()); 
			$(this).addClass("dirty");
		});
	    $(specifier + ".input-weight").change(function() {
		    objectConfig.setPartWeight(parseInt($(this).parents(".part-details").attr("id")), $(this).val()); 
		    $(this).addClass("dirty");
		});
	    $(specifier + ".input-via").change(function() {
		    objectConfig.setPartVia(parseInt($(this).parents(".part-details").attr("id")), $(this).val(), $(this).children(":selected").text()); 
		    $(this).addClass("dirty");
		});
	    $(specifier + ".input-origin").change(function() {
		    objectConfig.setPartOrigin(parseInt($(this).parents(".part-details").attr("id")), $(this).val(), true); 
		    $(this).addClass("dirty");
		});
	    $(specifier + ".input-rawemissions").change(function() {
		    objectConfig.setPartRawEmissions(parseInt($(this).parents(".part-details").attr("id")), $(this).val()); 
		    $(this).addClass("dirty");
		});
	    $(specifier + ".removeaction").click(function() {
		    objectConfig.removePart(parseInt($(this).parents(".part-details").attr("id")));
		});
	};

	/**
	 * Setup handlers for process
	 * @param specifier, this points to "li .part-details #id"
	 * Call the associated function in objects.js for every change function
	 * @see  #intializeEditMode
	 */
	this.setupProcessHandlers = function(specifier) {
           
	    $(".add-process").unbind("click");
	    $(".add-process-button").click(function(e){
					Sourcemap.Editor.addProcessPart(e.target);                        							     
		});
	    

	    $(specifier + ".input-process-area").change(function(){
			process_id =parseInt($(this).parents(".process-details").attr("id").substring(8));	
			var part_id = parseInt($(this).parents(".part-details").attr("id"));
			objectConfig.setPartProcessFactor(part_id, process_id, $(this).val(), 1);
		});

	    $(specifier + ".input-process-text").change(function(){
			process_id =parseInt($(this).parents(".process-details").attr("id").substring(8));
			var part_id = parseInt($(this).parents(".part-details").attr("id"));
			objectConfig.setPartProcessEmissions(part_id, process_id, $(this).val());
		});

	    $(specifier + ".input-process-display").change(function(){
			process_id =parseInt($(this).parents(".process-details").attr("id").substring(8));
			var part_id = parseInt($(this).parents(".part-details").attr("id"));
			objectConfig.setPartProcessName(part_id, process_id, $(this).val());		   
		});

	    $(specifier + ".process-unit").change(function(){
			process_id =parseInt($(this).parents(".process-details").attr("id").substring(8));
			var part_id = parseInt($(this).parents(".part-details").attr("id"));
			objectConfig.addProcessUnit(part_id, process_id, $(this).val());		   
		});

	    $(specifier + "span.removeprocessaction").click(function(){
			process_id =parseInt($(this).parents(".process-details").attr("id").substring(8));
			var part_id = parseInt($(this).parents(".part-details").attr("id"));
			flag = true;
			objectConfig.removePartProcess(part_id, process_id, flag);						   
		});	   

 
 	    
	};

	/**
	 * Setup handlers for endoflife
	 * @param specifier, this points to "li .part-details #id"
	 * Call the associated function in objects.js for every change function
	 * @see  #intializeEditMode
	 */
	this.setupEndoflifeHandlers = function(specifier) {
	    $(".add-endoflife").unbind("click");
	    $(".add-endoflife-button").click(function(e) {
					  Sourcemap.Editor.addEndoflifePart(e.target);
				      });
	    
	    $(specifier + ".input-endoflife-emissions").change(function(){
			endoflife_id =parseInt($(this).parents(".endoflife-details").attr("id").substring(10));
			var partId = parseInt($(this).parents(".part-details").attr("id"));
			objectConfig.setPartEndoflifeEmissions(partId, endoflife_id, $(this).val());
		});

	    $(specifier + ".input-endoflife-display").change(function(){
			endoflife_id =parseInt($(this).parents(".endoflife-details").attr("id").substring(10));
			var partId = parseInt($(this).parents(".part-details").attr("id"));
			objectConfig.setEndoflifeType(partId, endoflife_id, $(this).val());
		});	    

	    $(specifier + "span.removeendoflifeaction").click(function(){						
		        var id = parseInt($(this).parents(".part-details").attr("id"));
		        var endoflife_id = parseInt($(this).parents(".endoflife-details").attr("id").substring(10));
	                flag = true;

		        objectConfig.removePartEndoflife(id, endoflife_id, flag);
	       });

	};
	
	/**
	 * Get the value part-punch 
	 * @param id, part id
	 * Calculate totalval
	 */
	this.updatePartPunch = function(id) {		
	    var selectpunch = "#"+id+".part-details .part-punch-totalemissions";			    
	    var embodiedpunch = "#"+id+".part-details .part-punch-emissions";
	    var transportpunch = "#"+id+".part-details .part-punch-transportemissions";
	    var processpunch = "#"+id+".part-details .part-punch-processemissions";
	    var endoflifepunch = "#"+id+".part-details .part-punch-endoflifeemissions";
	    var embodiedval = $(embodiedpunch).text();
	    var transportval = $(transportpunch).text();
	    var processval = $(processpunch).text();
	    var endoflifeval = $(endoflifepunch).text();
	    if(objectConfig.showembodied == "") { embodiedval = 0;}
	    if(objectConfig.showtransport == "") { transportval = 0;}
	    if(objectConfig.showprocess == "") { processval = 0;}
	    if(objectConfig.showendoflife == "") { endoflifeval = 0;}
	    var totalval = Sourcemap.Util.roundIt(Number(embodiedval) + Number(transportval) + Number(processval) + Number(endoflifeval),2);
	    $(selectpunch).text("" + totalval);
	};
	
	/**
	 *  Receipt Toggle function for EmbodiedDisplay
	 * TransportDisplay, ProcessDisplay, EndoflifeDisplay
	 */
	this.toggleEmbodiedDisplay = function() {
	    if(objectConfig.showembodied == "on" && Sourcemap.Template.useembodied) { $(".part-rawemissions, .part-punch-emissions").css("display", "inline");}
	    else { $(".part-rawemissions, .part-punch-emissions").css("display", "none");}
	};
	
	this.toggleTransportDisplay = function() {
		if(objectConfig.showtransport == "on") { $(".part-shipping, .part-transport, .part-via, .part-punch-transportemissions").css("display", "inline");}
		else { $(".part-shipping, .part-transport, .part-via, .part-punch-transportemissions").css("display", "none");}
	};

	this.toggleProcessDisplay = function() {
		if(objectConfig.showprocess =="on" && Sourcemap.Template.useprocess) { $(".add-process, .process-list, .part-punch-processemissions").css("display", "inline"); }
		else { $(".add-process, .process-list, .part-punch-processemissions").css("display", "none"); }
	};
              
	this.toggleEndoflifeDisplay = function () {
		if(objectConfig.showendoflife == "on" && Sourcemap.Template.useendoflife) { $(".add-endoflife, .endoflife-list, .part-punch-endoflifeemissions").css("display", "inline"); }
		else { $(".add-endoflife, .endoflife-list, .part-punch-endoflifeemissions").css("display", "none"); }
	}; 
	
	// Currently unused
	this.initializeTooltips = function() {
	    $('.input-object-name').qtip({ content: '<div class="number">1</div>What is the name of your Sourcemap?', show: { when: false, ready: true }, hide: 'focus', position: { corner: { target: 'leftTop', tooltip: 'bottomLeft' } }, style: { width:540, 'font-size': 16, tip: 'bottomLeft', border: { width: 3, radius: 8, color: '#f9e98e' }, background: '#fbf7aa', 'font-weight': 'bold', color: '#000' } });
	    
	    $('.input-object-origin').qtip({ content: '<div class="number">2</div> What is the location?', show: { when: false, ready: true }, hide: 'focus', position: { corner: { target: 'rightMiddle', tooltip: 'topLeft' } }, style: { width:240, tip: 'topLeft', border: { width: 1, radius: 8, color: '#f9e98e' }, 'font-size': 12, background: '#fbf7aa', color: '#000' } });
	    
	    $('.input-object-description').qtip({ content: '<div class="number">3</div>You can add a description here, some html is ok.', show: { when: false, ready: true }, hide: 'focus', position: { corner: { target: 'bottomMiddle', tooltip: 'topMiddle' } }, style: { width:340, tip: 'topMiddle', border: { width: 1, radius: 8, color: '#f9e98e' }, 'font-size': 12, background: '#fbf7aa', color: '#000'} }); 
	};
	
	// Partlist visualization?
	this.partHighlight = function(id) {
	    $('#partlist-shell').scrollTo($("#"+id), 200, {offset:-10});
	    
	    $("#"+id+" .part-summary").css("background-color", "#ffffcc");
	    $("#"+id+" .part-summary").css("border-color", "#ffaa00");
	    $("#"+id).css("background-color", "#ffffcc");
	    $("#"+id).css("border-color", "#ffaa00");	    
	    $("#"+id+" .part-actions").css("background-color", "#ffaa00");
	    
	    $("#"+id+" .part-summary").animate({ backgroundColor: "#eeeeee", borderBottomColor: "#cccccc", borderTopColor: "#cccccc", borderLeftColor: "#cccccc", borderRightColor: "#cccccc"}, 3000 );
	    $("#"+id+" .part-summary").animate({ backgroundColor: "#eeeeee", borderBottomColor: "#cccccc", borderTopColor: "#cccccc", borderLeftColor: "#cccccc", borderRightColor: "#cccccc"}, 3000 );
	    $("#"+id).animate({ backgroundColor: "#eeeeee", borderBottomColor: "#cccccc", borderTopColor: "#cccccc", borderLeftColor: "#cccccc", borderRightColor: "#cccccc"}, 3000 );
	    $("#"+id).animate({ backgroundColor: "#f5f5f5", borderBottomColor: "#cccccc", borderTopColor: "#cccccc", borderLeftColor: "#cccccc", borderRightColor: "#cccccc"}, 3000 );
	    
	    $("#"+id+" .part-actions").animate({ backgroundColor: "#cccccc"}, 3000);	
	    
	    $("#"+id+".part-details").addClass("hover");
	    $("#"+id+" .list-expand").text("-"); 				
	};
	
	this.showPartChooser = function() {
	    SMap.clearPopup();
	    $('#partlist-shell, #partlist-menu').css("display","none");					
	    $("#part-shell, #part-visualization").css("width","125px");				
	    $('#partchooser-visualization').css("display", "block");
	    $("#partchooser-visualization .visualization-close-button").click(function() {Sourcemap.Editor.closePartChooser();});
	    $("#partlist-close-button").unbind("click");
	    $("#partlist-close-button").click(function() {
		    Sourcemap.Editor.closePartChooser();
		});			    			    
	};
	
	this.closePartChooser = function() {
	    $("#part-shell, #part-visualization").css("width", "540px"),
	    $('#partlist-menu').css("display","block");					
	    $('#partlist-shell').css("display","block"); 
	    $('#partchooser-visualization').css("display","none");	
	    $("#partlist-close-button").unbind("click");	
	    $("#partlist-close-button").click( function() { 
		    $("#part-visualization ").css("display", "none");
		});	
	};
	
	// Protopart definition

	this.protoPart = $.template('<li class="part-details hover" id="${pid}"><div class="part-actions"><span class="removeaction" style="display:inline">x</span>&nbsp;&nbsp;<span class="list-expand">-</span></div><div class="list-legend">${legend}</div><ul class="part-summary"><li class="part-name">${plinkstart}<input class="input-name" value="${pname}">${plinkfinish}</li><li class="part-origin"><input class="input-origin" value="${porigin}"></li><span class="list">+</span><li class="part-weight"><input class="input-weight" value="0"></li><span class="unit"><span class="part-punch-totalemissions">0</span></span><div style="clear:both;"></div></ul><div style="clear:both;"></div><ul class="part-properties"><li class="part-rawemissions"><input class="input-rawemissions" value="${pemissions}"></li><li class="part-punch-emissions">0</li><div style="clear:both;"></div><li class="part-shipping">0</li><li class="part-via">${pvia}</li><li class="part-punch-transportemissions">0</li><li class="part-punch-processemissions"><span class="unit">0</span></li><div style="clear:both;"></div><li class="part-punch-endoflifeemissions"><span class="unit">0</span></li><div style="clear:both;"></div></ul><div style="clear:both;"></div><div class="part-description"><textarea class="input-description">${pdescription}</textarea></div><div class="add-process"><div class="add-process-button">Add a Process</div> <select name="Processes" class="process-selected"> <option value="">None Selected</option></select></div><div class="clear"></div><div class="add-endoflife"> <div class="add-endoflife-button">Add End of Life</div><select name="Endoflife" class="endoflife-selected"> <option value="">None Selected</option> </select></div><div class="clear"></div><div class="process-list"> </div><div class="endoflife-list"></div><div class="clear"></div></li></div>');

	
	this.protoProcess = $.template('<div class="process-details" id="process-${id}"><span class="process-display"><input class="input-process-display" value="${name}"></span> @ <span class="process-text"><input class="input-process-text" value="${emissions}"></span> per <span class="process-area"><input class="input-process-area" value="${factor}"></span><span class="process-unit">${unit}</span> <span class="process-carbon"></span> <span class="removeprocessaction">x</span></div>');
	
	this.protoEndoflife = $.template('<div class="endoflife-details" id="endoflife-${id}"><input class="input-endoflife-display" value="${endoflife_type}"> @ <input class="input-endoflife-emissions" value="${emissions}"> per <span class="endoflife-unit">${endoflife_unit}</span> <span class="process-carbon"></span> <span class="removeendoflifeaction">x</span></div>');
	
	
	// Event callbacks
	this.receiveDescriptionChange = function(description) {
	    $(".input-object-description").val(description);
	};	
	
	/**
	 * Display the part using protoPart definition
	 * @param part id, name, description, emissions, link
	 * Call the setup handlers for process and endoflife
	 */
	this.receiveAddedPart = function(id, name, description, emissions, link) {	
	    $(".qtip").remove();
	    Sourcemap.Editor.closePartChooser();
	 	var linkurl =  Sourcemap.siteurl+link;
		if(linkurl != Sourcemap.siteurl) { var linkstart = '<a href="'+linkurl+'">'; var linkfinish = '</a>'; }
		else { var linkstart = ''; var linkfinish = '';}

	    $("#part-list").append( Sourcemap.Editor.protoPart , {
		    pid: id,
		    legend:id+1,
		    plinkstart:linkstart,
		    plinkfinish:linkfinish,
		    pname: name,
		    porigin: Sourcemap.Template.partlocationstubname,
		    pemissions: emissions,
		    pvia: $("#object-via").html(),
		    pdescription: description
		});

		Sourcemap.Editor.toggleEmbodiedDisplay();
		Sourcemap.Editor.toggleTransportDisplay();
		Sourcemap.Editor.toggleProcessDisplay();
		Sourcemap.Editor.toggleEndoflifeDisplay();
	    
		
		
		if(Sourcemap.Template.useprocess) {
			Sourcemap.Editor.addProcessPart($("li.#"+id+" .add-process-button"));		    
	    	Sourcemap.Editor.setupProcessHandlers("li.#"+id+" ");
		}
		if(Sourcemap.Template.useendoflife) {
			Sourcemap.Editor.addEndoflifePart($("li.#"+id+" .add-endoflife-button"));
	    	Sourcemap.Editor.setupEndoflifeHandlers("li.#"+id+" ");
		}
	
	    Sourcemap.Editor.setupLocationComplete("");

	    Sourcemap.Editor.setupPartHandlers("#"+id+" ");
	    if(	$('#part-visualization').css("display") == "none") {
		$('#part-visualization').css("display", "block");
	    }
	    $('#partlist-shell').scrollTo("max");	
		Sourcemap.Editor.partHighlight(id);
		
		$(".list-expand").bind("click", function() { 
			$(this).parents(".part-details").toggleClass("hover");
			if($(this).text() == "+") { $(this).text("-"); } else { $(this).text("+"); }
		});	
	};
	
	/**
	 * remove button visualization
	 * @param id, part id 
	 */
	this.receiveRemovedPart = function(id) {
		if(id == 0) { $(".part-details:first").remove(); } else {$("#"+id+".part-details").remove();}			    
	};
	
	// TODO Actually only the receipt cares about this...
	this.receiveObjectTotals = function(emissions, embodied, transport, process, endoflife, shipping, weight) {
	    $("#object-emissions").text(Sourcemap.Util.roundIt(Number(emissions),2));
	    $("#embodiedemissions-punch-value").text(Sourcemap.Util.roundIt(Number(embodied),2));
	    $("#transportemissions-punch-value").text(Sourcemap.Util.roundIt(Number(transport),2));
	    $("#object-shipping").text(Sourcemap.Util.roundIt(Number(shipping),2));
	    $("#object-weight").text(Sourcemap.Util.roundIt(Number(weight),2));		
	    $("#processemissions-punch-value").text(Sourcemap.Util.roundIt(Number(process),2));		
	    $("#endoflifeemissions-punch-value").text(Sourcemap.Util.roundIt(Number(endoflife),2));		
	    
	    for ( var part in objectConfig.parts ) { Sourcemap.Editor.updatePartPunch(part);}
	};
	
	this.receiveGeocodeError = function(element) {
	    $(element).parent().qtip({ content: "We couldn't find this location...", show: { when: false, ready: true }, hide: 'click', position: { corner: { target: 'topMiddle', tooltip: 'bottomMiddle' } }, style: { width:340, 'font-size': 16, tip: 'bottomMiddle', border: { width: 3, radius: 8, color: 'red' }, background: 'red', 'font-weight': 'bold', color: 'white'} });			    
	    setTimeout(function(){  $(".qtip:last").fadeOut("normal", function() {$(this).remove(); }); }, 2000);
	};
	
	this.receiveGeocodeMessage = function(element, address) {
	    $(element).val(address);
	    $(element).parent().qtip({ content: "We think you meant " + address + ".", show: { when: false, ready: true }, hide: 'click', position: { corner: { target: 'topMiddle', tooltip: 'bottomMiddle' } }, style: { width:340, 'font-size': 16, tip: 'bottomMiddle', border: { width: 3, radius: 8, color: '#f9e98e' }, background: '#fbf7aa', 'font-weight': 'bold',
color: '#000'} });			    
	    setTimeout(function(){  $(".qtip:last").fadeOut("normal", function() {$(this).remove(); }); }, 2000);
	};
	
	this.receiveSaveConfirmation = function(data) {
	    $( "#ajaxnote" ).text("");
	    window.onbeforeunload = null;
	    
	    if(data == "added") { 
		window.location.href = Sourcemap.siteurl+objectConfig.type+'/' + objectConfig.slug+"#edit";
	    }
	
		SMap.clearPopup();
		SMap.objectsummary = objectConfig;
		
		SMap.refreshMap();
	};
	                           
    // Part Event Handlers 	
	this.receivePartTotalsChanged = function(id) {
		Sourcemap.Editor.updatePartPunch(id);
	};
	
	this.receivePartNameChange = function(id, name) {
		$("#"+id+" .input-name").val(Sourcemap.Util.utf8decode(name));
	};
	
	this.receivePartDescriptionChange = function(id, description) {
		$("#"+id+" .input-description").val(Sourcemap.Util.utf8decode(description));
	};
	
	this.receivePartWeightChange = function(id, weight) {
		$("#"+id+" .input-weight").val(Number(weight));	
	};
	this.receivePartShippingChange = function(id, shipping) {
		$("#"+id+" .part-shipping").text(Sourcemap.Util.roundIt(Number(shipping),2));	
	};		
	this.receivePartEmissionsChange = function(id, emissions) {
		$("#"+id+" .part-punch-emissions").text(Sourcemap.Util.roundIt(Number(emissions),2));		
	};
	this.receievePartRawEmissionsChange = function(id, rawemissions) {
		$( "#"+id+" .part-rawemissions").val(Sourcemap.Util.roundIt(Number(rawemissions),2));
	};
	this.receivePartTransportEmissionsChange = function(id, transportemissions) {	
		$( "#"+id+" .part-punch-transportemissions").text("" + Sourcemap.Util.roundIt(Number(objectConfig.parts[id].shipping * (objectConfig.parts[id].weight/1000) * transportemissions),2));
	};
	this.receivePartViaChange = function(id, via) {
		$( "#"+id+" .input-via").val(via);	
	};
	this.receivePartOriginChange = function(id, origin) {
		$( "#"+id+" .input-origin").val(origin);	
		SMap.refreshMap();
	};

	// Process Event Handlers
	/**
	 * Display the endoflife using protoEndoflife definition
	 * @param partId, endlifelife_id, endoflife_type, emissions, unit
	 *  Call the setup handlers for endoflife
	 */
	this.receiveAddedProcess = function (id, processid, name, emissions, unit, flag, factor, check){
		$("#"+id+".part-details .process-list").append(Sourcemap.Editor.protoProcess,  {
			id: processid,
			name: name,
			emissions: emissions,
			unit: unit,
			factor: factor
		});

		if(unit == "kg") {
			var children = $("#"+id+".part-details .process-list").children();
			$(children[children.length-1]).find(".input-process-area").hide();
		}
		Sourcemap.Editor.setupProcessHandlers("li.#"+id+" ");
	};
	
	this.receiveProcessNameChange = function(id, processid, name) {
		$("#"+id+".part-details #process-"+processid+".process-details .input-process-display").val(Sourcemap.Util.utf8decode(name));  
	};
	this.receiveProcessEmissionsChange = function(id, processid, emissions) {
		$("#"+id+".part-details #process-"+processid+".process-details .input-process-text").text(Sourcemap.Util.roundIt(Number(emissions),2));  
	};
	this.receiveProcessUnitChange = function(id, processid, unit) {
		$("#"+id+".part-details #process-"+processid+".process-details .process-unit").val(Sourcemap.Util.utf8decode(unit));  
	};
	this.receiveProcessFlagChange = function(id, processid, flag) {
		$("#"+id+".part-details #process-"+processid+".flag").val(Sourcemap.Util.utf8decode(flag));  
	};
	this.receiveProcessFactorChange = function(id, factor, check) {
		if (check == 1) {		
			$("#"+id+".part-details #process"+processid+".process-details .input-process-area").val(Sourcemap.Util.utf8decode(factor));  
		}
	};	
	this.receiveRemovedProcess = function(id, processid) {
		if(processid == 0) { $("div.process-details:first").remove(); } 
		else { $("#"+id+".part-details #process-"+processid+".process-details").remove();}			    	    
	};
	this.receiveProcessCalculated = function(id, emissions) {
		$("#"+id+".part-details .part-punch-processemissions .unit").text(Sourcemap.Util.roundIt(Number(emissions),2));
	};
	
	// End of Life Event Handlers
	this.receiveEndoflifeType = function(partId, id, type) {
		$("#"+id+".endoflife-details .input-endoflife-display").val(Sourcemap.Util.utf8decode(type));  
	};

	this.receiveEndoflifeEmissions = function(partId, id, emissions) {
		$("#"+id+".endoflife-details .input-endoflife-emissions").val(Sourcemap.Util.utf8decode(emissions));  
	};

	this.receiveRemovedEndoflife = function(id, endoflife_id) {
	    if(endoflife_id == 0) { $("div.endoflife-details:first").remove(); } 
	    else { $("#"+id+".part-details #endoflife-"+endoflife_id+".endoflife-details").remove();}			    	    
	};
	this.receiveEndoflifeCalculated = function(id, emissions) {
		$("#"+id+".part-details .part-punch-endoflifeemissions .unit").text(Sourcemap.Util.roundIt(Number(emissions),2));
	};
	
	/**
	 * Display the endoflife using protoEndoflife definition
	 * @param partId, endlifelife_id, endoflife_type, emissions, unit
	 *  Call the setup handlers for endoflife
	 */
	this.receiveAddedEndoflife = function(partId, endoflife_id, endoflife_type, emissions, unit) {
            var endoflifeId = endoflife_id;
	    $("#"+partId+".part-details .endoflife-list").append(Sourcemap.Editor.protoEndoflife, {
			part_id: partId,
			id: endoflife_id,
			endoflife_type: endoflife_type,
			emissions: emissions,
			endoflife_unit: unit
		});

	    if(endoflife_type == "reuse") {
		var children = $("#"+partId+".part-details .endoflife-list").children();
		$(children[children.length-1]).find(".input-endoflife-emissions").hide();
		$(children[children.length-2]).find(".endoflife-unit").hide();
		
	    }

	    Sourcemap.Editor.setupEndoflifeHandlers("li.#"+partId+" ");
	
	};
	
	var slugToEndoflifeCache = Array();
	var slug_end = Array(100);
	var defaultSelectedTextEnd = "None Selected";
	var endoflife_id = 0;
	
	//Getting the endoflife list fro the parts id
	
	/**
	 * Getting the endoflife list for the parts id
	 * @param partId, endoflife_id, element
	 * Get the slug for the part and call saveEndoflifeForPart 
	 */
	this.getEndoflifeForPart = function(partId, endoflife_id, element) {
		var pslug = objectConfig.parts[partId].linkid;
	        var slug = pslug.split("/")[1];
		Sourcemap.Editor.saveEndoflifeForPart(slug, partId, endoflife_id);	    
	};
	
	/**
	 *  Display the endoflife dropdown array for the part
	 * @param slug_endoflife ,element, this is the part slug and target_element
	 */
	this.populateEndoflifeOptions = function(slug_endoflife, partId) {
		var options = "<option>"+defaultSelectedTextEnd+"</option>";
	  
		$.each(slug_endoflife, function(i,e) {
			options += '<option value="' + e.id + '">' + e.type + '</option>';   
			endoflife_id = e.id;
		});
		endoflife_id = endoflife_id+1;
		options += '<option value="' + endoflife_id + '">' + "reuse" + '</option>';   
           	    // If this element is a select element use set options.  Otherwise traverse up and set children.

		    $(".part-details#"+partId+" .endoflife-selected").html(options);		
	};
	
	/**
	 *  Get the JSON requet from ObjectsModel, if it already exists then cache it.
	 * @param slug_endoflife, partId, endoflife_id, element
	 */
	this.saveEndoflifeForPart = function(slug_endoflife, partId, endoflife_id) {
		slug_end = slug_endoflife;
		if(slugToEndoflifeCache[slug_end] != null) {
			Sourcemap.Editor.populateEndoflifeOptions(slugToEndoflifeCache[slug_end], partId);
		} else {
			var saveEndoflife = {parts_endoflife_slug: slug_endoflife};		
			var sendData = "data=" + JSON.stringify(saveEndoflife) + "";
			$.post(Sourcemap.siteurl+"objects/getEndoflife", sendData, function(response) {
				// depend on partId carrying through, should pass back with response
				$(".part-details#"+partId+" .endoflife-selected").css("display","inline");
				var objects_endoflife = eval(response); var options = "";
				slugToEndoflifeCache[slug_endoflife] = objects_endoflife;
				Sourcemap.Editor.populateEndoflifeOptions(slugToEndoflifeCache[slug_endoflife], partId);
			});
		}
	    
	    // Selecting an endoflife from the list 
		$(".endoflife-selected").unbind("change");
		$(".endoflife-selected").change(function() {
		    var endoflife_type = $(this).children("option:selected").text();

		    if(typeof(objectConfig.parts[partId].endoflife) == 'undefined') {
			endoflife_id =0;
		    } else {
			endoflife_id = $(objectConfig.parts[partId].endoflife).size();
		    }
	         					    

		    if(endoflife_type == defaultSelectedTextEnd) {return;}
						    
                    var endoflifeAlreadyAdded = false;
		    $("li.#"+partId+" .endoflife-list .input-endoflife-display").each(function(i, element) {
                       if(($(element).val() == endoflife_type) || (endoflife_id != 0)) {endoflifeAlreadyAdded = true; return false;}
			  return true;
			});
		    if(endoflifeAlreadyAdded) {return;}

						    
		    
		    var emissions = null;
		    var unit = null;
		    var unit_flag = false;
		    var flag = false;

		    $.each(slugToEndoflifeCache[slug_end], function(i, e) {
				if(e.type == endoflife_type) {
					emissions = e.emissions; unit = e.unit;
					if(unit != "kg") { unit_flag = true; }   
					return false;
				}
				return true;
			});
		   

		    objectConfig.addPartEndoflife(partId, endoflife_id, endoflife_type, emissions, unit, flag);
		});
	};


	

	//Getting the processes list for the parts id
	var slugToProcessCache = Array();
	var slug = Array(100);	
	var defaultSelectText = "None Selected:";
	var process_id = 0;
	var flag = false;
	var factor = 1;
	var check_factor = 0;
	
	/**
	 * Getting the processes list for the parts id
	 * @param part_id, process_id, elt
	 * call saveProcessForPart function 
	 */
	this.getProcessesForPart = function(part_id, process_id) {
	        var part_slug = objectConfig.parts[part_id].linkid;
	        var slug_part =part_slug.split("/")[1];
		Sourcemap.Editor.saveProcessForPart(slug_part, part_id, process_id);  
	};
	
	// Display the process array
	this.populateProcessOptions =function(slug_process, partid) {
	    var options ="<option>"+defaultSelectText+"</option>";	    
	    $.each(slug_process, function(i,e) { options +='<option value="' + e.id + '">' + e.name + '</option>';});	    
	    // If this element is a select element use set options.  Otherwise traverse up and set children.
		$(".part-details#"+partid+" .process-selected").html(options);		
	};
	
	//Get the JSON request from ObjectsModel, if it already exists then cache it.
	this.saveProcessForPart = function(data, part_id, process_id){
		slug = data;
		if(slugToProcessCache[slug] != null) {
			Sourcemap.Editor.populateProcessOptions(slugToProcessCache[slug], part_id);		
	    } else {		
			var saveData = { parts_slug : data };

			var sendData = "data=" + JSON.stringify(saveData) + "";
			$.post(Sourcemap.siteurl+"objects/getProcesses", sendData, function(response) {
				// depends on partID carrying through, should change.
				$(".part-details#"+part_id+" .process-selected").css("display","inline");
				var objects = eval(response);
				var options = "";
				slugToProcessCache[slug] = objects;
				Sourcemap.Editor.populateProcessOptions(slugToProcessCache[slug], part_id);
			});
		}

		//Selecting a process from the list and if it exists in cache then 
		$(".process-selected").unbind("change");
		$(".process-selected").change(function() {
		    var process_name = $(this).children("option:selected").text();

                
		    if(typeof(objectConfig.parts[part_id].process) == "undefined") {
			process_id = 0;
		    } else {
                        process_id = $(objectConfig.parts[part_id].process).size();
		    }
		    

		    if(process_name == defaultSelectText) { return;}

		    var processAlreadyAdded = false;
		    $(".process-list .input-process-display").each( function(i, element) {
			    if($(element).val() == process_name) { processAlreadyAdded = true; return false;}
			    return true;
			});
		    if(processAlreadyAdded) {return;}
		    
		    // Get emissions and unit from cache
		    var emissions = null;
		    var unit = null;
		    var unit_flag = false;
		 
		    $.each(slugToProcessCache[slug], function(i,e) { 
				if(e.name == process_name) {
					emissions = e.emissions;
					unit = e.unit;
					if (unit != "kg"){ unit_flag = true; }
					return false;
				}
				return true;
			});
		    
		    if (unit_flag == true) { check_factor = 1; factor = 1; } 
			else{ factor = 0; check_factor = 0;}
						  

		    objectConfig.addPartProcess(part_id, process_id, process_name, emissions, unit, flag, factor, check_factor);		   		    
		});
	};

	
	// Misc Functions TODO These functions should probably be moved somewhere else...	
	this.calculateUsage = function() {
	    var factorVal = $("#poweruselist").find(":selected").val();
	    var usage = objectConfig.usageenergy * factorVal;
	    $("#usepunch").text(Sourcemap.Util.roundIt(Number(usage),2));
	    objectConfig.usageemissions = Sourcemap.Util.roundIt(Number(usage),2);
	};
	
	/**
	 * setting up click functions set up the permissions 
	 * call save function to save the permission settings on the server
	 */			
	this.setupPermissions = function() {
		$(".permission_user").click(function() {
			Sourcemap.Editor.savePermissions("useredit");
			Sourcemap.Editor.removePermissions();
			$(".permission_user").addClass("selected");							    
		});
	    $("#groupedit").change(function() {
			Sourcemap.Editor.savePermissionsGroups($("#groupedit option:selected").text());
			Sourcemap.Editor.removePermissions();
			$(".permission_group").addClass("selected");
		});
	    $(".permission_group").click(function() {
			Sourcemap.Editor.savePermissionsGroups($("#groupedit option:selected").text());
			Sourcemap.Editor.removePermissions();
			$(".permission_group").addClass("selected");
		});
	    
	    $(".permission_everyone").click(function() {
			Sourcemap.Editor.savePermissions("everyoneedit");
			Sourcemap.Editor.removePermissions();
			$(".permission_everyone").addClass("selected");
			$(".permission_public").addClass("selected");
			$(".permission_private").removeClass("selected");
		});
	    $(".permission_public").click(function() {
			objectConfig.visibility = "public";
			Sourcemap.Editor.savePermissionsObjects("public");
			Sourcemap.Editor.removeVisibility();
			$(".permission_public").addClass("selected");
		});
		$(".permission_private").click(function() {
			if($(".permission_public").hasClass("selected") && $(".permission_everyone").hasClass("selected")){ }
			else {
				objectConfig.visibility = "private";
				Sourcemap.Editor.savePermissionsObjects("private");
				Sourcemap.Editor.removeVisibility();
				$(".permission_private").addClass("selected");
			}
		});
	};
	
	/**
	 * Remove function visualizations
	 */		
	this.removeVisibility = function() {
		$(".permission_public").removeClass("selected");
		$(".permission_private").removeClass("selected");			
	};

	this.removePermissions = function() {
		$(".permission_user").removeClass("selected");
		$(".permission_group").removeClass("selected");
		$(".permission_everyone").removeClass("selected");
	};
		/**
		 * call objects controller/model to save the settings
		 */
       this.savePermissions = function(data) {
		var saveData = {
		    permission_type: data,
		    oid: objectConfig.oid
		};
	
		var sendData = "data=" + JSON.stringify(saveData) + "";
		$.post(Sourcemap.siteurl+"objects/setpermissions", sendData);			
    };

    this.savePermissionsGroups = function(data) {
		var saveData = {
			group_name: data,
			permission_type: "groupedit",
		    oid: objectConfig.oid
		};
	
		var sendData = "data=" + JSON.stringify(saveData) + "";
		$.post(Sourcemap.siteurl+"objects/setpermissions", sendData);			
    };
    
    this.savePermissionsObjects = function(permission) {
		var saveData = {
		    visibility: permission,
		    oid: objectConfig.oid
		};
	
		var sendData = "data=" + JSON.stringify(saveData) + "";
		$.post(Sourcemap.siteurl+"objects/setvisibility", sendData);
	};		

	// Listen for object events
	$("body").bind('objectDescriptionUpdated', function(e, description) {
		Sourcemap.Editor.receiveDescriptionChange(description); 
	});
	$("body").bind('originGeoError', function(e, id) {
		if(id == "object") { Sourcemap.Editor.receiveGeocodeError(".input-object-origin"); }
		else { Sourcemap.Editor.receiveGeocodeError("#"+id+" .input-origin"); }
	});
	$("body").bind('originGeoMessage', function(e, id, msg) {
		if(id == "object") { Sourcemap.Editor.receiveGeocodeMessage(".input-object-origin", msg); }
		else { Sourcemap.Editor.receiveGeocodeMessage("#"+id+" .input-origin", msg); }
	});	
	$("body").bind("objectTotalsCalculated", function(e, emis, embod, trans, process, end, ship, weight) {	
		Sourcemap.Editor.receiveObjectTotals(emis,embod,trans,process,end,ship,weight);
	});			
	$("body").bind("saveObjectConfirmation", function(e, data) { Sourcemap.Editor.receiveSaveConfirmation(data);});	
	$("body").bind("partAdded", function(e, id, name, description, emissions, link) { 
		Sourcemap.Editor.receiveAddedPart(id, name, description, emissions, link);
	});	
	$("body").bind("partRemoved", function(e, id) {
		Sourcemap.Editor.receiveRemovedPart(id);
		SMap.refreshMap();
	});	
			
	// Listen for part events
	$("body").bind("partTotalsChanged", function(e, id) {
		Sourcemap.Editor.receivePartTotalsChanged(id);
	});	
	$("body").bind("partNameUpdated", function(e, id, name) {
		Sourcemap.Editor.receivePartNameChange(id, name);
	});	
	$("body").bind("partDescriptionUpdated", function(e, id, description) {
		Sourcemap.Editor.receivePartDescriptionChange(id, description);
	});
	$("body").bind("partWeightUpdated", function(e, id, weight) {
		Sourcemap.Editor.receivePartWeightChange(id, weight);		
	});
	$("body").bind("partShippingUpdated", function(e, id, shipping) {
		Sourcemap.Editor.receivePartShippingChange(id, shipping);		
	});	
	$("body").bind("partEmissionsUpdated", function(e, id, emissions) {
		Sourcemap.Editor.receivePartEmissionsChange(id, emissions);
	});	
	$("body").bind("partRawEmissionsUpdated", function(e, id, rawemissions) {
		Sourcemap.Editor.receievePartRawEmissionsChange(id, rawemissions);
	});
	$("body").bind("partTransportEmissionsUpdated", function(e, id, transportemissions) {
		Sourcemap.Editor.receivePartTransportEmissionsChange(id, transportemissions);
	});
	$("body").bind("partViaUpdated", function(e, id, via) {
		Sourcemap.Editor.receivePartViaChange(id, via);
	});
	$("body").bind("partOriginUpdated", function(e, id, origin) {
		Sourcemap.Editor.receivePartOriginChange(id, origin);
	});	

	$("body").bind("partSelected", function(e, id) {
		Sourcemap.Editor.partHighlight(id-1);
	});

	// Listen for process events
	$("body").bind("partProcessAdded", function(e, id, processid, name, emissions, unit, flag, factor, check) { 
		Sourcemap.Editor.receiveAddedProcess(id, processid, name, emissions, unit, flag, factor, check);
	});
	$("body").bind("partProcessNameUpdated", function(e, id, processid, name){
		Sourcemap.Editor.receiveProcessNameChange(id, processid, name);	   
	});
	$("body").bind("partProcessEmissionsUpdated", function(e, id, processid, emissions){
		Sourcemap.Editor.receiveProcessEmissionsChange(id, processid, emissions);	   
	});
	$("body").bind("partProcessUnitUpdated", function(e, id, processid, unit){
		Sourcemap.Editor.receiveProcessUnitChange(id, processid, unit);	   
	});
	$("body").bind("partProcessFlagUpdated", function(e, id, processid, flag){
		Sourcemap.Editor.receiveProcessFlagChange(id, processid, flag);	   
	});
	$("body").bind("partProcessFlagUpdated", function(e, id, processid, factor, check){
		Sourcemap.Editor.receiveProcessFactorChange(id, processid, factor, check);	   
	});
	$("body").bind("partProcessRemoved", function(e, id, processid) {
		Sourcemap.Editor.receiveRemovedProcess(id, processid);
	});	
	$("body"). bind("partProcessCalculated", function (e, id, emissions) {
		Sourcemap.Editor.receiveProcessCalculated(id, emissions); 
	});
	
	// Listen for endoflife events
	$("body").bind("partEndoflifeAdded", function(e, id, endid, endtype, emissions, unit) {
		Sourcemap.Editor.receiveAddedEndoflife(id, endid, endtype, emissions, unit); 
	});
	$("body").bind("partEndoflifeRemoved", function(e, id, endid) {
		Sourcemap.Editor.receiveRemovedEndoflife(id, endid);
	});
	$("body"). bind("partEndoflifeEmissionsUpdated", function (e, id, endid, emissions) {
		Sourcemap.Editor.receiveEndoflifeEmissions(id, endid, emissions); 
	});
	$("body"). bind("partEndoflifeTypeUpdated", function (e, id, endid, type) {
		Sourcemap.Editor.receiveEndoflifeType(id, endid, type); 
	});
	$("body"). bind("partEndoflifeCalculated", function (e, id, emissions) {
		Sourcemap.Editor.receiveEndoflifeCalculated(id, emissions); 
	});
	
	// === UI Setup
	$("#more-description-button").click( function() { 
		$("body").scrollTo($("#object-story"), 500);
	});
			
	// Visualization Menu
	$("#partlist-close-button").click( function() { 
		$("#part-visualization").slideToggle("normal",function(){
			SMap.fitMap($("#part-visualization").css("display")=="block");
		});
	});
	$(".visualization-close-button").click( function() { 
		$(".visualization").css("display", "none");
	});
	$(".comment-visualization-button").click( function() { 
		$(".visualization").css("display", "none");
		$("#comment-visualization").slideDown();
	});
	$(".charting-visualization-button").click( function() { 
		$(".visualization").css("display", "none");
		$("#charting-visualization").slideDown();
	    });
	$(".partlist-visualization-button").click( function() { 
		$("#part-visualization").slideToggle("normal",function(){
			SMap.fitMap($("#part-visualization").css("display")=="block");
		});
	});
					
	// Editing Interface
	$("#editsourcemapbutton").click( function() { 
		Sourcemap.Editor.initializeEditMode();
		if(Sourcemap.Template.useprocess) {
			$(".add-process").show();
		}
		if(Sourcemap.Template.useendoflife) {		
			$(".add-endoflife").show();
		}
	});
	$("#savesourcemapbutton").click( function() { 
		Sourcemap.Editor.stopEditMode(); 
		if(Sourcemap.Template.useprocess) {
			$(".add-process").hide();
		}
		if(Sourcemap.Template.useendoflife) {		
			$(".add-endoflife").hide();
		}
	});
			
	if(!(Sourcemap.Template.usechooser)) { $("#addpartbutton, #partlist-menu").click( function() { objectConfig.addPart(Sourcemap.Template.partstubname, '', '0', ''); });} 
	else { $("#addpartbutton, #partlist-menu").click( function() { Sourcemap.Editor.showPartChooser(); });}
	
	// Actions
	$("#permission-button").click( function() { 
		$("#delivered-panel").slideUp("fast"); $("#lastleg-button").removeClass("selected");				
		$("#share-panel").slideUp("fast"); $("#share-button").removeClass("selected");			
		$("#permission-panel").slideToggle("fast"); $("#permission-button").toggleClass("selected");
	});
	$("#share-button").click( function() { 
		$("#delivered-panel").slideUp("fast"); $("#lastleg-button").removeClass("selected");
		$("#permission-panel").slideUp("fast"); $("#permission-button").removeClass("selected");						
		$("#share-panel").slideToggle("fast"); $("#share-button").toggleClass("selected");				
	});
	$("#lastleg-button").click( function() { 
		$("#share-panel").slideUp("fast"); $("#share-button").removeClass("selected");				
		$("#permission-panel").slideUp("fast"); $("#permission-button").removeClass("selected");												
		$("#delivered-panel").slideToggle("fast"); $("#lastleg-button").toggleClass("selected");
		
	});
	
	// Map setup
	var smapOptions = {};
	if ($('#part-visualization').css("display") != "none")
	{
		var pv = $('#part-visualization');
		smapOptions.constrain = pv[0].clientWidth + '_rc';
	}
	
	smapOptions.userLocation = false;
	smapOptions.navPosition = new OpenLayers.Pixel(2,15);
	
	// Initialize Map
	SMap = new Sourcemap.OpenSourcemap( 'smap', objectConfig, smapOptions );
	if(GBrowserIsCompatible()){ SMap.geoCoder = new GClientGeocoder(); }			

	// Toggles Setup
	$(".list-expand").bind("click", function() { 
		$(this).parents(".part-details").toggleClass("hover");
		if($(this).text() == "+") { $(this).text("-"); } else { $(this).text("+"); }
	});
									
	$("#showfootprint").click(function() {
		if(objectConfig.showfootprint != "on") { objectConfig.showfootprint = "on";} else { objectConfig.showfootprint = "";}		
		if(objectConfig.showfootprint == "on") { checkval = true;} else { checkval = false;}
		$("#showembodied").attr('checked', checkval);
		objectConfig.showembodied = objectConfig.showfootprint;
		$("#showtransport").attr('checked', checkval);
		objectConfig.showtransport = objectConfig.showfootprint;
		$("#showprocess").attr('checked', checkval);
		objectConfig.showprocess = objectConfig.showfootprint;
		$("#showendoflife").attr('checked', checkval);
		objectConfig.showendoflife = objectConfig.showfootprint;
		
		$("#showuse").attr('checked', checkval);
		objectConfig.showuse = objectConfig.showfootprint;
		objectConfig.recalculateObjectValues();
		Sourcemap.Editor.toggleEmbodiedDisplay();
		Sourcemap.Editor.toggleTransportDisplay();
		Sourcemap.Editor.toggleProcessDisplay();
		Sourcemap.Editor.toggleEndoflifeDisplay();
	});
			
	$("#showembodied").click(function() { if(objectConfig.showembodied != "on") { objectConfig.showembodied = "on";} else { objectConfig.showembodied = "";} objectConfig.recalculateObjectValues(); Sourcemap.Editor.toggleEmbodiedDisplay();});
	$("#showtransport").click(function() { if(objectConfig.showtransport != "on") { objectConfig.showtransport = "on";} else { objectConfig.showtransport = "";} objectConfig.recalculateObjectValues(); Sourcemap.Editor.toggleTransportDisplay();});
	$("#showprocess").click(function() { if(objectConfig.showprocess != "on") { objectConfig.showprocess = "on";} else { objectConfig.showprocess = "";} objectConfig.recalculateObjectValues(); Sourcemap.Editor.toggleProcessDisplay();});
	$("#showendoflife").click(function() { if(objectConfig.showendoflife != "on") { objectConfig.showendoflife = "on";} else { objectConfig.showendoflife = "";} objectConfig.recalculateObjectValues(); Sourcemap.Editor.toggleEndoflifeDisplay();});
	$("#showuse").click(function() { if(objectConfig.showuse != "on") { objectConfig.showuse = "on";} else { objectConfig.showuse = "";} objectConfig.recalculateObjectValues();});
	$("#showstats").click(function() { if(objectConfig.showstats != "on") { objectConfig.showstats = "on";} else { objectConfig.showstats = "";} objectConfig.recalculateObjectValues();});
	
	$("#usekwh").change(function() {
		objectConfig.usageenergy = $(this).val();
		Sourcemap.Editor.calculateUsage();
	});
	$("#poweruselist").change(function() {
		var selectName = $(this).parent().find(":selected").text();
		objectConfig.usagetype = selectName;
		Sourcemap.Editor.calculateUsage();
	});
			
	// Live events
	$(".input-name").live("click", function(){ if($(this).val() == Sourcemap.Template.partstubname) { $(this).val(""); } return false;});
	$(".input-origin").live("click", function(){ if($(this).val() == Sourcemap.Template.partlocationstubname) { $(this).val(""); } return false;});	

	// Additional UI
	$("#part-visualization, .visualization").draggable({ containment: '#map_container', revert: true, handle: "#part-shell-menu, .visualization-menu" });			
	if(Sourcemap.Template.usereceipt) {			 
		$("#summary-statistics div").css("display","block");	
		$("#summary-statistics").accordion();
	}
    
	objectConfig.recalculateObjectValues();			
	Sourcemap.Editor.processHashEvent(window.location.hash);	        		

	}
};
    
var SMap = null;

// Initial call (setup for edit mode)
$(document).ready(function() {		
	Sourcemap.Editor.init();
});
